﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Windows.Controls;

namespace TestProject
{
    /// <summary>
    /// 컨트롤 메뉴 항목
    /// </summary>
    public class ControlMenuItem : MenuItem
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ControlMenuItem()

        /// <summary>
        /// 생성자
        /// </summary>
        public ControlMenuItem()
        {
            Assembly assembly = Assembly.GetAssembly(typeof(Control));

            Type[] typeArray = assembly.GetTypes();

            SortedList<string, MenuItem> menuItemSortedList = new SortedList<string, MenuItem>();

            Header = "Control";
            Tag    = typeof(Control);

            menuItemSortedList.Add("Control", this);

            foreach(Type type in typeArray)
            {
                if(type.IsPublic && (type.IsSubclassOf(typeof(Control))))
                {
                    MenuItem menuItem = new MenuItem();

                    menuItem.Header = type.Name;
                    menuItem.Tag    = type;

                    menuItemSortedList.Add(type.Name, menuItem);
                }
            }

            foreach(KeyValuePair<string, MenuItem> keyValuePair in menuItemSortedList)
            {
                if(keyValuePair.Key != "Control")
                {
                    string praentTypeName = ((Type)keyValuePair.Value.Tag).BaseType.Name;

                    MenuItem parentMenuItem = menuItemSortedList[praentTypeName];

                    parentMenuItem.Items.Add(keyValuePair.Value);
                }
            }

            foreach(KeyValuePair<string, MenuItem> keyValuePair in menuItemSortedList)
            {
                Type type = (Type)keyValuePair.Value.Tag;

                if(type.IsAbstract && keyValuePair.Value.Items.Count == 0)
                {
                    keyValuePair.Value.IsEnabled = false;
                }

                if(!type.IsAbstract && keyValuePair.Value.Items.Count > 0)
                {
                    MenuItem menuItem = new MenuItem();

                    menuItem.Header = keyValuePair.Value.Header as string;
                    menuItem.Tag    = type;

                    keyValuePair.Value.Items.Insert(0, menuItem);
                }
            }
        }

        #endregion
    }
}